import logging
import sys


def get_logger(name: str = "scribesearch", level: int = logging.INFO) -> logging.Logger:
    """
    Configure and return a logger instance.
    """
    logger = logging.getLogger(name)

    if not logger.handlers:  # avoid duplicate handlers
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter(
            "[%(asctime)s] [%(levelname)s] %(name)s: %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        logger.setLevel(level)

    return logger



import os
import tempfile

from scribe_search.exceptions import InvalidSource, ScribeSearchException

from .srt import Loader as SrtLoader

try:
    import ffmpeg
except ImportError:
    raise ScribeSearchException("Please install ffmpeg-python to use this")


class Loader:

    @staticmethod
    def load(sources: list[str]):
        for path in sources:
            # We should fail early before models take over
            if not os.path.isfile(path):
                raise InvalidSource(f"File {path} not found")
            try:
                probe = ffmpeg.probe(path)
                if not [s for s in probe["streams"] if s["codec_type"] == "subtitle"]:
                    raise InvalidSource(f"No subtitles found in video {path}")
            except Exception:
                raise InvalidSource("Error probing video")

        tmp_paths = []
        for path in sources:
            probe = ffmpeg.probe(path)
            subtitle_streams = [s for s in probe["streams"] if s["codec_type"] == "subtitle"]

            base_name = os.path.basename(path)

            for i, _stream in enumerate(subtitle_streams):

                tmp = tempfile.NamedTemporaryFile(prefix=f"{base_name}_sub{i}_", suffix=".srt", delete=False)
                tmp_path = tmp.name
                tmp.close()

                ffmpeg.input(path).output(tmp_path, map=f"0:s:{i}", **{"c:s": "srt"}).run(
                    quiet=True, overwrite_output=True
                )

                if os.path.exists(tmp_path):
                    tmp_paths.append(tmp_path)

        subs = list(SrtLoader.load(tmp_paths))

        for path in tmp_paths:
            os.remove(path)

        return subs



from importlib import import_module
from typing import Iterable, Literal

from scribe_search.data import Result, Sub


def load_source(source_type: Literal["srt", "youtube", "video"], sources: list[str]) -> Iterable[Sub]:
    Loader = getattr(import_module(f"scribe_search.loaders.{source_type}"), "Loader")
    return Loader.load(sources)


def scribe_search(
    source_type: Literal["srt", "youtube", "video"], sources: list[str], query: str, top: int = 5
) -> Iterable[Result]:

    subs = list(load_source(source_type, sources))

    # faster
    from scribe_search.search import build_faiss_index, make_sentence_chunks, semantic_search

    chunks = make_sentence_chunks(subs)
    index, _embeddings, texts = build_faiss_index(chunks)

    return semantic_search(query, index, texts, chunks, top)



from typing import Iterable

import nltk
from numpy import ndarray

try:
    nltk.data.find("tokenizers/punkt")
except LookupError:
    nltk.download("punkt")

try:
    nltk.data.find("tokenizers/punkt_tab")
except LookupError:
    nltk.download("punkt_tab")

import faiss
from nltk.tokenize import sent_tokenize
from sentence_transformers import SentenceTransformer

from scribe_search.data import Chunk, Result, Sub

model = SentenceTransformer("all-MiniLM-L6-v2")


def _flush_sentences(buffer_text: str) -> tuple[list, str]:
    sentences = sent_tokenize(buffer_text)

    # If only one sentence, it might still be incomplete → keep buffering
    if len(sentences) == 1:
        return [], buffer_text

    return sentences[:-1], sentences[-1]


def make_sentence_chunks(subs: Iterable[Sub]) -> list[Chunk]:
    """
    From a list of Subs, return a list of Chunks. A chunk is a group of
    sentences in a time interval
    """

    chunks = []

    buffer_text = ""
    buffer_start = None
    end_sub = None

    for sub in subs:
        text = sub.text.replace("\n", " ").strip()

        # Initialize start timestamp if buffer is empty
        if buffer_start is None:
            buffer_start = sub.start

        buffer_text += " " + text if buffer_text else text

        sentences, flushed = _flush_sentences(buffer_text)

        for sentence in sentences:
            chunks.append(
                Chunk(
                    text=sentence.strip(),
                    start=buffer_start,
                    end=sub.end,
                    source=sub.source,
                    source_type=sub.source_type,
                )
            )

        if flushed:
            buffer_text = flushed
            buffer_start = sub.end
        end_sub = sub

    # Catch leftover text (in case transcript ends without punctuation)
    if buffer_text and buffer_start:
        chunks.append(
            Chunk(
                text=buffer_text.strip(),
                start=buffer_start,
                end=end_sub.end,
                source=end_sub.source,
                source_type=end_sub.source_type,
            )
        )

    return chunks


def build_faiss_index(chunks: Iterable[Chunk]) -> tuple[faiss.Index, ndarray, list]:
    """
    Takes list of sentence chunks (with text, start, end)
    and builds a FAISS index.
    """
    texts = [c.text for c in chunks]
    embeddings = model.encode(texts, convert_to_numpy=True, normalize_embeddings=True)

    dim = embeddings.shape[1]
    index = faiss.IndexFlatIP(dim)  # cosine similarity if normalized
    index.add(embeddings)

    return index, embeddings, texts


def semantic_search(query: str, index: faiss.Index, texts: list[str], chunks: list[Chunk], top_k: int = 5):
    q_emb = model.encode([query], convert_to_numpy=True, normalize_embeddings=True)
    scores, idxs = index.search(q_emb, top_k)

    results = []
    for score, idx in zip(scores[0], idxs[0]):
        c = chunks[idx]
        results.append(
            Result(
                chunk=c,
                score=score,
                index=idx,
            )
        )

    return results



import sys

from scribe_search.exceptions import ScribeSearchException
from scribe_search.utils import scribe_search

from .cli import parse_args


def main():
    args = parse_args()

    source = None
    try:
        results = scribe_search(args.source_type, args.source, args.query, args.top)
    except ScribeSearchException as e:
        print(e)
        sys.exit(1)

    for result in results:
        if result.chunk.source != source:
            source = result.chunk.source
            print()
            print(result.chunk.source)
            print(len(result.chunk.source) * "*")
        print(result)


if __name__ == "__main__":
    main()



import mimetypes
import os
import re

from scribe_search.exceptions import InvalidSource
from scribe_search.loaders import LoaderInterface

from .srt import Loader as SRTLoader
from .video import Loader as VideoLoader
from .youtube import Loader as YTLoader


class Loader(LoaderInterface):

    @staticmethod
    def get_loader(source: str) -> type:
        if re.search(r"(https?://)?(www\.)?(youtube\.com|youtu\.be)/", source):
            # Youtube link
            return YTLoader

        if os.path.exists(source):
            mime_type, _ = mimetypes.guess_type(source)
            if mime_type:
                if mime_type.startswith("video/"):
                    return VideoLoader
                return SRTLoader

        raise InvalidSource(f"Can't infer the source type from `{source}`")

    @staticmethod
    def load(sources: list[str]):
        for source in sources:
            loader_class = Loader.get_loader(source)
            return loader_class.load([source])



import re
from typing import Iterable

from youtube_transcript_api import YouTubeTranscriptApi
from youtube_transcript_api.formatters import SRTFormatter

from scribe_search.data import Sub
from scribe_search.exceptions import InvalidSource

from .srt import Loader as SrtLoader


class Loader:

    @staticmethod
    def extract_video_id(url: str) -> str:

        if re.match(r"^[a-zA-Z0-9_-]{11}$", url):
            return url

        patterns = [
            r"(?:https?://)?(?:www\.)?youtube\.com/watch\?v=([a-zA-Z0-9_-]{11})",
            r"(?:https?://)?(?:www\.)?youtu\.be/([a-zA-Z0-9_-]{11})",
            r"(?:https?://)?(?:www\.)?youtube\.com/embed/([a-zA-Z0-9_-]{11})",
            r"(?:https?://)?(?:www\.)?youtube\.com/v/([a-zA-Z0-9_-]{11})",
        ]

        for pattern in patterns:
            match = re.search(pattern, url)
            if match:
                return match.group(1)

        raise InvalidSource(f"Could not extract video ID from URL: {url}")

    @staticmethod
    def load(sources: list[str]) -> Iterable[Sub] | None:
        api = YouTubeTranscriptApi()
        formatter = SRTFormatter()

        for source in sources:
            video_id = Loader.extract_video_id(source)
            try:
                caption = api.fetch(video_id)
            except Exception as e:
                raise InvalidSource(f"Could not fetch transcripts for video {source}: {str(e)}")

            caption = formatter.format_transcript(caption)
            return SrtLoader.load([caption], source_type="youtube", source_override=source)



from dataclasses import dataclass
from datetime import time

TIME_FORMAT = "%H:%M:%S"


@dataclass
class Chunk:
    start: time
    end: time
    text: str
    source: str
    source_type: str

    @property
    def start_h(self):
        return self.start.strftime(TIME_FORMAT)

    @property
    def end_h(self):
        return self.end.strftime(TIME_FORMAT)

    def __str__(self):
        return f"{self.start_h} - {self.end_h}: {self.text}"


@dataclass
class Sub(Chunk): ...


@dataclass
class Result:
    chunk: Chunk
    score: float
    index: int

    def __str__(self):
        return f"{self.chunk} x {self.score:.3f}"



import os
from datetime import time
from typing import Iterable

import webvtt
import webvtt.srt
from webvtt.errors import MalformedCaptionError

from scribe_search.data import Sub
from scribe_search.exceptions import InvalidSource
from scribe_search.loaders import LoaderInterface


class Loader(LoaderInterface):

    @staticmethod
    def load(sources: list[str], source_type="srt", source_override=None) -> Iterable[Sub]:
        for source in sources:
            try:
                if os.path.isfile(source):
                    captions = webvtt.from_srt(source)
                else:
                    captions = webvtt.srt.parse(source.splitlines())
            except MalformedCaptionError as e:
                raise InvalidSource from e

        for source in sources:
            for caption in captions:
                yield Sub(
                    start=time(*caption.start_time.to_tuple()),
                    end=time(*caption.end_time.to_tuple()),
                    text=caption.text,
                    source=source_override or source,
                    source_type=source_type,
                )



import argparse


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="ScribeSearch: Semantic search over SRT, YouTube, or local videos.")
    parser.add_argument(
        "--source-type",
        "-t",
        type=str,
        choices=["srt", "youtube", "video", "auto"],
        default="auto",
        help="Source type, srt file, youtube or local video. Defaults to auto and tries to infer the source.",
    )
    parser.add_argument("source", type=str, nargs="+", help="YouTube URLs, path to .srt files, or local video files")
    parser.add_argument("--query", "-q", type=str, help="Semantic search query")
    parser.add_argument("--top", "-n", type=int, default=5, help="Number of top results to return")
    return parser.parse_args()
